/**
 * @ngdoc directive
 * @id input
 * @name ons-input
 * @category form
 * @description
 *  [en]Input component.[/en]
 *  [ja]inputコンポ―ネントです。[/ja]
 * @codepen ojQxLj
 * @guide UsingFormComponents
 *   [en]Using form components[/en]
 *   [ja]フォームを使う[/ja]
 * @guide EventHandling
 *   [en]Event handling descriptions[/en]
 *   [ja]イベント処理の使い方[/ja]
 * @example
 * <ons-input></ons-input>
 * <ons-input modifier="material" label="Username"></ons-input>
 */

/**
 * @ngdoc attribute
 * @name label
 * @type {String}
 * @description
 *   [en]Text for animated floating label.[/en]
 *   [ja]アニメーションさせるフローティングラベルのテキストを指定します。[/ja]
 */

/**
 * @ngdoc attribute
 * @name float
 * @description
 *  [en]If this attribute is present, the label will be animated.[/en]
 *  [ja]この属性が設定された時、ラベルはアニメーションするようになります。[/ja]
 */

/**
 * @ngdoc attribute
 * @name ng-model
 * @extensionOf angular
 * @description
 *   [en]Bind the value to a model. Works just like for normal input elements.[/en]
 *   [ja]この要素の値をモデルに紐付けます。通常のinput要素の様に動作します。[/ja]
 */

/**
 * @ngdoc attribute
 * @name ng-change
 * @extensionOf angular
 * @description
 *   [en]Executes an expression when the value changes. Works just like for normal input elements.[/en]
 *   [ja]値が変わった時にこの属性で指定したexpressionが実行されます。通常のinput要素の様に動作します。[/ja]
 */

(function(){
  'use strict';

  var ATTRS = [
    'ngModel',
    'ngChange',
    'ngRequired',
    'ngMinlength',
    'ngMaxlength',
    'ngPattern',
    'ngTrim',
    'ngValue',
    'ngTrueValue',
    'ngFalseValue'
  ];

  angular.module('onsen').directive('onsInput', function($compile) {
    return {
      restrict: 'E',
      replace: false,
      scope: false,

      link: function(scope, element, attrs) {
        CustomElements.upgrade(element[0]);

        var el = element[0];
        var type = el.getAttribute('type');

        ATTRS.forEach(function(attr) {
          var kebabCase = attr.replace(/[A-Z]/g, function(letter, pos) {
            return (pos ? '-' : '') + letter.toLowerCase();
          });

          if (attrs.hasOwnProperty(attr)) {
            el._input.setAttribute(kebabCase, attrs[attr]);
          }
        });

        $compile(el._input)(scope);

        if (el._isTextInput && attrs.ngModel) {
          scope.$watch(attrs.ngModel, function(value) {
            el._updateLabelClass();
          });
        }

        scope.$on('$destroy', function() {
          scope = element = attrs = el = null;
        });
      }
    };
  });
})();
